       IDENTIFICATION DIVISION.
       PROGRAM-ID.    CBLRMPC1.
       AUTHOR.        SIMOTIME TECHNOLOGIES.
      *****************************************************************
      * Copyright (C) 1987-2019 SimoTime Technologies.                *
      *                                                               *
      * All rights reserved.  Unpublished, all rights reserved under  *
      * copyright law and international treaty.  Use of a copyright   *
      * notice is precautionary only and does not imply publication   *
      * or disclosure.                                                *
      *                                                               *
      * Permission to use, copy, modify and distribute this software  *
      * for any non-commercial purpose and without fee is hereby      *
      * granted, provided the SimoTime copyright notice appear on all *
      * copies of the software. The SimoTime name or Logo may not be  *
      * used in any advertising or publicity pertaining to the use    *
      * of the software without the written permission of SimoTime    *
      * Technologies.                                                 *
      *                                                               *
      * Permission to use, copy, modify and distribute this software  *
      * for any commercial purpose requires a fee to be paid to       *
      * SimoTime Technologies. Once the fee is received by SimoTime   *
      * the latest version of the software will be delivered and a    *
      * license will be granted for use within an enterprise,         *
      * provided the SimoTime copyright notice appear on all copies   *
      * of the software. The SimoTime name or Logo may not be used    *
      * in any advertising or publicity pertaining to the use of the  *
      * software without the written permission of SimoTime           *
      * Technologies.                                                 *
      *                                                               *
      * SimoTime Technologies makes no warranty or representations    *
      * about the suitability of the software for any purpose. It is  *
      * provided "AS IS" without any expressed or implied warranty,   *
      * including the implied warranties of merchantability, fitness  *
      * for a particular purpose and non-infringement. SimoTime       *
      * Technologies shall not be liable for any direct, indirect,    *
      * special or consequential damages resulting from the loss of   *
      * use, data or projects, whether in an action of contract or    *
      * tort, arising out of or in connection with the use or         *
      * performance of this software                                  *
      *                                                               *
      * SimoTime Technologies                                         *
      * 15 Carnoustie Drive                                           *
      * Novato, CA 94949-5849                                         *
      * 415.883.6565                                                  *
      *                                                               *
      * RESTRICTED RIGHTS LEGEND                                      *
      * Use, duplication, or disclosure by the Government is subject  *
      * to restrictions as set forth in subparagraph (c)(1)(ii) of    *
      * the Rights in Technical Data and Computer Software clause at  *
      * DFARS 52.227-7013 or subparagraphs (c)(1) and (2) of          *
      * Commercial  Computer Software - Restricted Rights  at 48      *
      * CFR 52.227-19, as applicable.  Contact SimoTime Technologies, *
      * 15 Carnoustie Drive, Novato, CA 94949-5849.                   *
      *                                                               *
      *****************************************************************
      *      This program is provided by SimoTime Technologies        *
      *        Our e-mail address is: helpdesk@simotime.com           *
      *     Also, visit our Web Site at http://www.simotime.com       *
      *                                                               *
      *****************************************************************
      *
      *****************************************************************
      * Source Member: CBLRMPC1.CBL
      * Copy Files     PASSPARS.CPY
      * Calls to:      SIMOPARS
      *****************************************************************
      *
      * CBLRMPC1 - Call SIMOPARS to parse the buffer.
      *
      * CALLING PROTOCOL
      * ----------------
      * Use standard procedure to EXECUTE, RUN or ANIMATE.
      *
      * DESCRIPTION
      * -----------
      * This is a demonstration program to show how to call the
      * COBOL Parsing Routine Routine.
      *
      *          ************
      *          * CBLRMPJ1 *
      *          ********jcl*
      *               *
      *               *
      *          ************     ************
      *          * CBLRMPC1 *-----* CONSOLE  *
      *          ********cbl*     ******dsply*
      *               *
      *               *
      *          ************
      *          * SIMOPARS *
      *          ********cbl*
      *
      *****************************************************************
      *
      * MAINTENANCE
      * -----------
      * 1989/02/27 Simmons, Created program.
      * 1997/03/17 Simmons, Updated for COBOL/2.
      * 2002/09/23 Simmons, Added function to create a new field with
      *            leading spaces removed and embedded, multiple spaces
      *            reduced to a single space.  The length of the output
      *            text string is also calculated.
      *
      *****************************************************************
      *
       ENVIRONMENT DIVISION.
       DATA DIVISION.
       WORKING-STORAGE SECTION.
      *****************************************************************
      *    Data-structure for Title and Copyright...
      *    ------------------------------------------------------------
       01  SIM-TITLE.
           05  T1 pic X(11) value '* CBLRMPC1 '.
           05  T2 pic X(34) value 'COBOL Parse a Data String         '.
           05  T3 pic X(10) value ' v03.12.02'.
           05  T4 pic X(24) value ' http://www.simotime.com'.
       01  SIM-COPYRIGHT.
           05  C1 pic X(11) value '* CBLRMPC1 '.
           05  C2 pic X(20) value 'Copyright 1987-2019 '.
           05  C3 pic X(28) value '   SimoTime Technologies    '.
           05  C4 pic X(20) value ' All Rights Reserved'.

       01  SIM-THANKS-01.
           05  C1 pic X(11) value '* CBLRMPC1 '.
           05  C2 pic X(32) value 'Thank you for using this program'.
           05  C3 pic X(32) value ' provided from SimoTime Technolo'.
           05  C4 pic X(04) value 'gies'.

       01  SIM-THANKS-02.
           05  C1 pic X(11) value '* CBLRMPC1 '.
           05  C2 pic X(32) value 'Please send all inquires or sugg'.
           05  C3 pic X(32) value 'estions to the helpdesk@simotime'.
           05  C4 pic X(04) value '.com'.

      *****************************************************************
      *    Buffer used for posting messages to the console.
      *    ------------------------------------------------------------
       01  MESSAGE-BUFFER.
           05  MESSAGE-HEADER      pic X(11)   value '* CBLRMPC1 '.
           05  MESSAGE-TEXT        pic X(245).

      *****************************************************************
      *    Work fields used for testing call to SIMOPARS.
      *    ------------------------------------------------------------

       01  IX-1                    pic 9999 value 0.
       01  IX-2                    pic 9999 value 0.

      *    Simple parsing task...
       01  TASK-01-DATA.
           05  filler pic X(30) value 'This is a simple text string  '.

      *    Used to parse using a slash (/) as the delimiter...
       01  TASK-02-DATA.
           05  filler pic X(10) value '01/24/2001'.
           05  filler pic X(5)  value is SPACES.

      *    Used to parse and handle the leading and/or embedded
      *    SPACE or multiple SPACES...
       01  TASK-03-DATA.
           05  filler pic X(07) value is SPACES.
           05  filler pic X(30) value 'Text   String with leading    '.
           05  filler pic X(30) value 'and multiple   embedded       '.
           05  filler pic X(05) value is SPACES.
           05  filler pic X(20) value 'SPACE   characters  '.

      *    Parse a larger data string...
       01  TASK-04-DATA.
           05  filler pic X(30) value '  The quick brown fox jumped  '.
           05  filler pic X(30) value 'over the lazy dogs back. Now  '.
           05  filler pic X(30) value 'is the time for all good men  '.
           05  filler pic X(30) value 'and women to come to the aid  '.
           05  filler pic X(30) value 'of the party.                 '.
           05  filler pic X(30) value 'We have crossed the bridge and'.
           05  filler pic X(30) value '   we are pushing the envelope'.
           05  filler pic X(30) value ' to the   top of the   hill.  '.

       01  DATA-256        pic X(256)  value SPACES.
       01  WORK-256        pic X(256)  value SPACES.
       01  DATA-256-MESSAGE.
           05  filler  pic X(29) value 'Length of the output text is '.
           05  DATA-256-LENGTH   pic 9(4)    value 0.

       01  LAST-BYTE-BUFFER.
           05  filler      pic X(25) value 'Last significant byte is '.
           05  LAST-BYTE   pic 9(4)  value 0.

       COPY PASSPARS.
      *****************************************************************
       PROCEDURE DIVISION.

           perform Z-POST-COPYRIGHT.

      *!   Do a simple parsing to identify the position and size
      *!   of words within an alphameric field.
           perform SAMPLE-TASK-01.

      *!   Do a simple parsing to identify the position and size
      *!   of the mm, dd and ccyy within a date field.
           perform SAMPLE-TASK-02.

      *!   Do the parsing to identify the position and size of the
      *!   words within an alphameric field. Also, create a new
      *!   field with the leading spaces removed and the embedded,
      *!   multiple spaces reduced to a single space.
           perform SAMPLE-TASK-03.

      *!   Do the parsing to identify the position and size of the
      *!   words within an alphameric field. Also, create a new
      *!   field with the leading spaces removed and the embedded,
      *!   multiple spaces reduced to a single space. This is similar
      *!   to the preceding sample but parses a larger field.
           perform SAMPLE-TASK-04.

           perform Z-THANK-YOU.

           GOBACK.

      *****************************************************************
      * Scan a data string and identify the starting position and
      * length of each keyword in the string, The delimiter used for
      * this task is a space character.
      *****************************************************************
       SAMPLE-TASK-01.
      *!   Display the start-of-task message.
           move 'SAMPLE-TASK-01 is STARTING...' to MESSAGE-TEXT
           perform Z-POST-MESSAGE
      *!   Display the input buffer.
           move TASK-01-DATA to MESSAGE-TEXT
           perform Z-POST-MESSAGE
      *!   Prepare control items for parsing.
           move '0' to PRS-REQUEST
           add 9 to ZERO giving PRS-STATUS
           move ' ' to PRS-DELIMITER
      *    move 'N' to PRS-TERMINATOR-FLAG
           move ' ' to PRS-TERMINATOR-BYTE
           add 32 to ZERO giving PRS-TABLE-MAX
           add 128 to ZERO giving PRS-BUFFER-SIZE
      *!   Move the data string to the parsing buffer and call
      *!   the parsing routine.
           move TASK-01-DATA to    PRS-BUFFER
           call 'SIMOPARS'   using PRS-PARAMETERS
      *!   Display the contents of the parsing tables.
           add 1 to ZERO giving IX-1
           perform until PRS-SIZE(IX-1) = 0
                      or PRS-POSITION(IX-1) = 0
                      or IX-1       > PRS-TABLE-MAX
               perform Z-POST-TABLE-ITEM
               add 1 to IX-1
           end-perform
           perform Z-POST-LAST-SIGNIFICANT-BYTE
      *!   Display the end-of-task message.
           move 'SAMPLE-TASK-01 is COMPLETE...' to MESSAGE-TEXT
           perform Z-POST-MESSAGE
           exit.

      *****************************************************************
      * Scan a data string and identify the starting position and
      * length of each keyword in the string, The delimiter used for
      * this task is a slash (/) character.
      * This example also uses a space character to terminate the
      * parsing if the string.
      *****************************************************************
       SAMPLE-TASK-02.
      *!   Display the start-of-task message.
           move 'SAMPLE-TASK-02 is STARTING...' to MESSAGE-TEXT
           perform Z-POST-MESSAGE
      *!   Display the input buffer.
           move TASK-02-DATA to MESSAGE-TEXT
           perform Z-POST-MESSAGE
      *!   Prepare control items for parsing.
           move '0' to PRS-REQUEST
           add 9 to ZERO giving PRS-STATUS
           move '/' to PRS-DELIMITER
      *    move 'Y' to PRS-TERMINATOR-FLAG
           move ' ' to PRS-TERMINATOR-BYTE
           add 32 to ZERO giving PRS-TABLE-MAX
           add 128 to ZERO giving PRS-BUFFER-SIZE
      *!   Move the data string to the parsing buffer and call
      *!   the parsing routine.
           move TASK-02-DATA to PRS-BUFFER
           call 'SIMOPARS'     using PRS-PARAMETERS
      *!   Display the contents of the parsing tables.
           add 1 to ZERO giving IX-1
           perform until PRS-SIZE(IX-1) = 0
                      or PRS-POSITION(IX-1) = 0
                      or IX-1             > PRS-TABLE-MAX
               perform Z-POST-TABLE-ITEM
               add 1 to IX-1
           end-perform
           perform Z-POST-LAST-SIGNIFICANT-BYTE
      *!   Display the end-of-task message.
           move 'SAMPLE-TASK-02 is COMPLETE...' to MESSAGE-TEXT
           perform Z-POST-MESSAGE
           exit.

      *****************************************************************
      * Scan a data string and identify the starting position and
      * length of each keyword in the string, The delimiter used for
      * this task is a space character.
      * The input data string in this task has leading and multiple,
      * embedded spaces.
      * The output data string has the leading spaces removed and the
      * embedded multiple spaces are replaced with a single space.
      *****************************************************************
       SAMPLE-TASK-03.
      *!   Display the start-of-task message.
           move 'SAMPLE-TASK-03 is STARTING...' to MESSAGE-TEXT
           perform Z-POST-MESSAGE
      *!   Display the input buffer.
           move 'Input with leading and embedded, multiple spaces...'
             to MESSAGE-TEXT
           perform Z-POST-MESSAGE
      *!   Display the input buffer.
           move TASK-03-DATA to MESSAGE-TEXT
           perform Z-POST-MESSAGE
      *!   Prepare control items for parsing.
           move '0' to PRS-REQUEST
           add 9 to ZERO giving PRS-STATUS
           move ' ' to PRS-DELIMITER
      *    move 'N' to PRS-TERMINATOR-FLAG
           move ' ' to PRS-TERMINATOR-BYTE
           add 32 to ZERO giving PRS-TABLE-MAX
           add 128 to ZERO giving PRS-BUFFER-SIZE
      *!   Move the data string to the parsing buffer and call
      *!   the parsing routine.
           move TASK-03-DATA to PRS-BUFFER
           call 'SIMOPARS'     using PRS-PARAMETERS
      *!   Display the contents of the parsing tables.
           add 1 to ZERO giving IX-1
           perform until PRS-SIZE(IX-1) = 0
                      or PRS-POSITION(IX-1) = 0
                      or IX-1       > PRS-TABLE-MAX
               perform Z-POST-TABLE-ITEM
               add 1 to IX-1
           end-perform
           perform Z-POST-LAST-SIGNIFICANT-BYTE
      *!   Create and display the output buffer.
           move 'Output w/o leading and embedded, multiple spaces...'
             to MESSAGE-TEXT
           perform Z-POST-MESSAGE
           move TASK-03-DATA to WORK-256
           perform SAMPLE-TASK-99
           move DATA-256-MESSAGE to MESSAGE-TEXT
           perform Z-POST-MESSAGE
           move DATA-256 to MESSAGE-TEXT
           perform Z-POST-MESSAGE
      *!   Display the end-of-task message.
           move 'SAMPLE-TASK-03 is COMPLETE...' to MESSAGE-TEXT
           perform Z-POST-MESSAGE
           exit.

      *****************************************************************
      * Scan a data string and identify the starting position and
      * length of each keyword in the string, The delimiter used for
      * this task is a space character.
      * The input data string in this task has leading and multiple,
      * embedded spaces.
      * The output data string has the leading spaces removed and the
      * embedded multiple spaces are replaced with a single space.
      *****************************************************************
       SAMPLE-TASK-04.
      *!   Display the start-of-task message.
           move 'SAMPLE-TASK-04 is STARTING...' to MESSAGE-TEXT
           perform Z-POST-MESSAGE
      *!   Display the input buffer.
           move 'Input with leading and embedded, multiple spaces...'
             to MESSAGE-TEXT
           perform Z-POST-MESSAGE
      *!   Display the input buffer.
           move TASK-04-DATA to MESSAGE-TEXT
           perform Z-POST-MESSAGE
      *!   Prepare control items for parsing.
           move '0' to PRS-REQUEST
           add 9 to ZERO giving PRS-STATUS
           move ' ' to PRS-DELIMITER
      *    move 'N' to PRS-TERMINATOR-FLAG
           move ' ' to PRS-TERMINATOR-BYTE
           add 64 to ZERO giving PRS-TABLE-MAX
           add 240 to ZERO giving PRS-BUFFER-SIZE
      *!   Move the data string to the parsing buffer and call
      *!   the parsing routine.
           move TASK-04-DATA to PRS-BUFFER
           call 'SIMOPARS'     using PRS-PARAMETERS
      *!   Display the contents of the parsing tables.
           add 1 to ZERO giving IX-1
           perform until PRS-SIZE(IX-1) = 0
                      or PRS-POSITION(IX-1) = 0
                      or IX-1       > PRS-TABLE-MAX
               perform Z-POST-TABLE-ITEM
               add 1 to IX-1
           end-perform
           perform Z-POST-LAST-SIGNIFICANT-BYTE
           move TASK-04-DATA to MESSAGE-TEXT
           perform Z-POST-MESSAGE
      *!   Create and display the output buffer.
           move 'Output w/o leading and embedded, multiple spaces...'
             to MESSAGE-TEXT
           perform Z-POST-MESSAGE
           move SPACES       to WORK-256
           move TASK-04-DATA to WORK-256
           perform SAMPLE-TASK-99
           move DATA-256-MESSAGE to MESSAGE-TEXT
           perform Z-POST-MESSAGE
           move DATA-256-MESSAGE to MESSAGE-TEXT
           move DATA-256 to MESSAGE-TEXT
           perform Z-POST-MESSAGE
      *!   Display the end-of-task message.
           move 'SAMPLE-TASK-04 is COMPLETE...' to MESSAGE-TEXT
           perform Z-POST-MESSAGE
           exit.

      *****************************************************************
      * Build a new string with a single space between words and
      * remove the leading spaces.
      *****************************************************************
       SAMPLE-TASK-99.
           add 1 to ZERO giving IX-1
           add 1 to ZERO giving IX-2
           move SPACES to DATA-256
           perform until PRS-SIZE(IX-1) = 0
                      or PRS-POSITION(IX-1) = 0
                      or IX-1       > PRS-TABLE-MAX
               move WORK-256(PRS-POSITION(IX-1):PRS-SIZE(IX-1))
                 to DATA-256(IX-2:PRS-SIZE(IX-1))
               add PRS-SIZE(IX-1) to IX-2
               add 1 to IX-2
               add 1 to IX-1
           end-perform
           if  IX-2 > 1
               subtract 2 from IX-2 giving DATA-256-LENGTH
           else
               add 1 to ZERO giving DATA-256-LENGTH
           end-if

           exit.

      *****************************************************************
       Z-POST-COPYRIGHT.
           display SIM-TITLE
           display SIM-COPYRIGHT
           exit.

       Z-POST-LAST-SIGNIFICANT-BYTE.
           add PRS-LAST-SIG-BYTE to ZERO giving LAST-BYTE
           move LAST-BYTE-BUFFER to MESSAGE-TEXT
           perform Z-POST-MESSAGE
           exit.

       Z-POST-MESSAGE.
           if  MESSAGE-BUFFER(80:177) = SPACES
               display MESSAGE-BUFFER(1:79)
               move SPACES to MESSAGE-TEXT
           else
               display MESSAGE-BUFFER
               move SPACES to MESSAGE-TEXT
           end-if
           exit.

       Z-POST-TABLE-ITEM.
           move 'Item-nnnn, Position-nnnn, Size-nnnn, Parameter - '
             to MESSAGE-TEXT(1:49)
           move IX-1 to MESSAGE-TEXT(6:4)
           move PRS-POSITION(IX-1) to MESSAGE-TEXT(21:4)
           move PRS-SIZE(IX-1) to MESSAGE-TEXT(32:4)
           if  PRS-SIZE(IX-1) < 20
               move PRS-BUFFER(PRS-POSITION(IX-1):PRS-SIZE(IX-1))
                 to MESSAGE-TEXT(50:PRS-SIZE(IX-1))
           else
               move PRS-BUFFER(PRS-POSITION(IX-1):19)
                 to MESSAGE-TEXT(50:19)
           end-if
           perform Z-POST-MESSAGE
           exit.

      *****************************************************************
       Z-THANK-YOU.
           display SIM-THANKS-01
           display SIM-THANKS-02
           exit.
      *****************************************************************
      *      This example is provided by SimoTime Technologies        *
      *        Our e-mail address is: helpdesk@simotime.com           *
      *     Also, visit our Web Site at http://www.simotime.com       *
      *****************************************************************
